-- packages/generic_trees/sql/generic-trees-tables.sql
--
-- @creation-date March 2001
--
-- @author Aram Kananov (akananov@arsdigita.com)
--
-- @cvs-id $Id: generic-trees-tables.sql,v 1.1 2001/04/27 10:03:05 arcanan Exp $
--
-- TODO List:
-- 1) add indexes to improve perfomance
-- 2) give the names to all constraints and indexes

create table generic_tree_types (
  type_id	     integer
		     constraint generic_tree_types_pk 
		     primary key
		     constraint generic_tree_types_id_fk 
		     references acs_objects(object_id),
  type_name	     varchar2(50) not null 
		     constraint generic_tree_type_name_un
		     unique,
  pretty_name	     varchar2(50) not null 
		     constraint generic_tree_pretty_name_un
		     unique,
  pretty_plural	     varchar2(50) not null
		     constraint generic_tree_pretty_plural_un 
		     unique,
  tree_struct_table  varchar2(50) not null 
		     constraint generic_tree_tree_table_un
		     unique,
  default_function_no  integer,		     		     
  generated_p	     char(1) default 'f' not null
		     constraint generic_tree_types_gener_p_ck
		     check (generated_p in ('t', 'f')), 
  owner_id	     integer		     
);


comment on table generic_tree_types is '
 This table stores meta data about a generic tree type. All instances of a 
 tree type are stored in the same database table. Each
 tree type can have a set of Oracle functions for retrieving information
 (typically name) of the application data that is in the tree.
'; 

comment on column generic_tree_types.tree_struct_table is '
 The name of the database table where the node structure of this tree type is
 stored. This table is created by the generic_tree_type PL/SQL package. Every node
 in the tree will have an object_id (a reference to acs_objects).
';

comment on column generic_tree_types.generated_p is '
 Indicates if the database table for storing the tree structure has been 
 created';


begin  
  acs_object_type.create_type ( 
    supertype     => 'acs_object', 
    object_type	  => 'generic_tree_type', 
    pretty_name	  => 'Generic tree type', 
    pretty_plural => 'Generic tree types', 
    table_name    => 'GENERIC_TREE_TYPES', 
    id_column	  => 'TYPE_ID',
    name_method	  => 'generic_tree_type.name'
  ); 
end;
/
show errors


create table generic_tree_functions (
  type_id	    integer not null
		    constraint generic_tree_functions_id_fk 
		    references generic_tree_types,
  purpose	    varchar2(150) not null,
  pretty_name	    varchar2(50) not null 
		    constraint generic_tree_functions_p_n_un
		    unique,
  pretty_plural	    varchar2(50) not null 
		    constraint generic_tree_functions_p_p_un
		    unique,
  function_no	    integer default 1 not null,
  ind_expr	    varchar2(150) not null,
  index_name	    varchar2(50) not null 
		    constraint generic_tree_functions_i_n_un
		    unique, 
  constraint	    generic_tree_functions_pk 
  primary key (type_id, function_no)
); 

comment on table generic_tree_functions is '
 Table for storing Oracle functions belonging to a certain tree type (every tree
 type has its own table to store its structure and object_id:s).
 These functions are used to retrieve node labels (i.e. category names) or other 
 application data associated with the object_id:s hanging in the tree. Every function
 should have an associated functional index for fast retrievals. It is the responsibility
 of the applications to create these indices.
'; 

comment on column generic_tree_functions.function_no is '
 A unique id for the function for quick look up.
';

comment on column generic_tree_functions.ind_expr is '
 The function expression to be used in an SQL query to fetch
 the label/name (i.e. ac_category.short_name(object_id) ).
';

comment on column generic_tree_functions.index_name is '
 The name of the database index used to enable the usage 
 of the function in a SQL statement to be fast.
';


create table generic_trees (
   tree_id               integer not null 
		         constraint generic_trees_pk
		         primary key
		         constraint generic_trees_id_fk 
		         references acs_objects(object_id),
   type_id               integer not null 
			 constraint generic_trees_type_id_fk
			 references generic_tree_types,
   tree_name		 varchar2(50) not null 
			 constraint generic_trees_type_name_un
			 unique,
   description		 varchar2(1024),
   support_nested_set_p  char(1) default 'f'
			 constraint generic_trees_nested_p_ck
			 check (support_nested_set_p in ('t', 'f')), 
   default_function_no	 integer,
   root_node_id		 integer
);

comment on table generic_trees is '
 Table for storing meta data about tree instances of different types. The actual tree
 structure is stored in a table (with the name specified in generic_tree_types) 
 generated by the generic tree API.
';


comment on column generic_trees.support_nested_set_p is '
 We use nested sets to optimize our SQL queries.
';

comment on column generic_trees.default_function_no is '
 The function that will be used per default to retrieve labels for the
 tree.
';


begin  
  acs_object_type.create_type ( 
    supertype     => 'acs_object', 
    object_type	  => 'generic_tree', 
    pretty_name	  => 'Generic tree', 
    pretty_plural => 'Generic trees', 
    table_name    => 'GENERIC_TREES', 
    id_column	  => 'Tree_ID',
    name_method	  => 'generic_tree.name'
  ); 
end;
/

create sequence generic_tree_node_id start with 1;
